unit Perlin;

{Usage:
 First of all you should initialize all values (the standard values won't satisfy you :-) )
 For example:
  SetPersistence(0.6);
  SetSeed(12345);
  SetDepth(4);
 The amplitude depends on your application. For RGB values you usually require:
  SetAmplitude(0.0, 255.0);
 After you've called Create you can start producing values on demand. Make sure you
 call the GetValue function with an interval of about 0.01 depending on your application. If
 you use an interval of >= 1 you won't notice any smoothing!
}

interface

uses math, sysutils;

type
  TPerlinNoise2D = class
  protected
    m_Seed: integer;
    m_Depth: integer;
    m_Amplitude: Single;
    m_Add: Single;
    m_Persistence: Single;

    function Interpolate(V1, V2, Step: Single): Single;
    function InterpolatedRand(X, Y: Single): Single;
    function SmoothRand(X, Y: Integer): Single;
    function Rand(X, Y: Integer): Single;

  public
    procedure SetStandardValues;

    // Set the minimal and maximal values for the result
    procedure SetAmplitude(Min, Max: Single);

    // Set the persistence of the perlin noise
    procedure SetPersistence(Persistence: Single);

    // Set the random generator seed for the perlin noise
    // This is not the common random generator seed because this class has it's own
    // random generator. Seed can have any value.
    procedure SetSeed(Seed: Integer);

    // Set the calculation depth for the perlin noise
    procedure SetDepth(Depth: Integer);

    // Calculate additional values from the set parameters
    // This function has to be called for applying some of the Set*-functions.
    constructor Create;

    // Get a value for a given X and Y coordinate
    function GetValue(X, Y: Single): Single;

  end;

implementation

procedure TPerlinNoise2D.SetStandardValues;
begin
  m_Seed  := 0;
  m_Depth := 5;

  m_Add       := 0;
  m_Amplitude := 1;
end;

procedure TPerlinNoise2D.SetAmplitude(Min, Max: Single);
begin
  m_Add       := Min;
  m_Amplitude := Max;
end;

procedure TPerlinNoise2D.SetPersistence(Persistence: Single);
begin
  m_Persistence := Persistence;
end;

procedure TPerlinNoise2D.SetSeed(Seed: Integer);
begin
  m_Seed := Seed;
end;

procedure TPerlinNoise2D.SetDepth(Depth: Integer);
begin
  m_Depth := Depth;
end;

constructor TPerlinNoise2D.Create;
var
  L_Range: Single;
  L_SummedMultiplyer: Single;
  L_Multiplyer: Single;
  L_Count: Integer;
begin
  inherited Create;

  SetStandardValues;

  L_Range := m_Amplitude - m_Add;
  m_Add := m_Add + L_Range * 0.5;

  // We get values between -1 and 1 out of the Rand function
  L_SummedMultiplyer := 0;
  L_Multiplyer       := 2;

  for L_Count := 0 to m_Depth -1 do
  begin
    L_SummedMultiplyer := L_SummedMultiplyer + L_Multiplyer;
    L_Multiplyer := L_Multiplyer * m_Persistence;
  end;

  m_Amplitude := L_Range / L_SummedMultiplyer;
end;

function TPerlinNoise2D.GetValue(X, Y: Single): Single;
var
  L_Total: Single;
  L_Amplitude: Single;
  L_Count: Integer;
begin
  L_Total := 0;
  L_Amplitude := m_Amplitude;

  for L_Count := 0 to m_Depth -1 do
  begin
    L_Total := L_Total + InterpolatedRand(X, Y) * L_Amplitude;

    L_Amplitude := L_Amplitude * m_Persistence;

    X := X * 2;
    Y := Y * 2;

    // Without this we get a strong precision loss for large values and also overflows.
    // This can't also prevent this forever but it's better than nothing.
    // A modulo operation is too time consuming but maybe a while(.. >= .. ) does a
    // better job.
    if X >= 4096 then X := X - 4096;

    if Y >= 4096 then Y := Y - 4096;
  end;

  L_Total := L_Total + m_Add;

  result := L_Total;
end;

function TPerlinNoise2D.InterpolatedRand(X, Y: Single): Single;
var
  L_IntX: Integer;
  L_FracX: Single;

  L_IntY: Integer;
  L_FracY: Single;

  L_V1, L_V2, L_V3, L_V4, L_ResX1, L_ResX2: Single;
begin
  L_IntX  := Trunc(X);
  L_FracX := X - L_IntX;

  L_IntY  := Trunc(Y);
  L_FracY := Y - L_IntY;

  L_V1 := SmoothRand(L_IntX,     L_IntY    );
  L_V2 := SmoothRand(L_IntX + 1, L_IntY    );
  L_V3 := SmoothRand(L_IntX,     L_IntY + 1);
  L_V4 := SmoothRand(L_IntX + 1, L_IntY + 1);

  L_ResX1 := Interpolate(L_V1, L_V2, L_FracX);
  L_ResX2 := Interpolate(L_V3, L_V4, L_FracX);

  result := Interpolate(L_ResX1, L_ResX2, L_FracY);
end;

function TPerlinNoise2D.Interpolate(V1, V2, Step: Single): Single;
var
  L_Fact: Single;
begin
  // This interpolation methode is faster than a cosinus function, but it's neary
  // equal to the cos-graph. I simply take a square function either in positiv
  // direction or in negativ direction starting from 1.0.

  if( Step > 0.5 ) then
  begin
    L_Fact := (1.0 - Step);
    L_Fact := L_Fact * L_Fact * 2.0;
  end
  else
  begin
    L_Fact := 1.0 - Step * Step * 2.0;
  end;

  result := V1 * L_Fact + V2 * (1.0 - L_Fact);
end;

function TPerlinNoise2D.SmoothRand(X, Y: Integer): Single;
begin
  result := Rand(X, Y);

  // The following code gives a smoother perlin noise, but it's also quite slow
{var
  L_Corners: Single;
  L_Sides: Single;
  L_Center: Single;
begin
  L_Corners := ( Rand(X-1, Y-1) + Rand(X+1, Y-1) + Rand(X-1, Y+1) + Rand(X+1, Y+1) ) * ( 1.0 / 16.0 );
  L_Sides   := ( Rand(X-1, Y  ) + Rand(X+1, Y  ) + Rand(X  , Y-1) + Rand(X  , Y+1) ) * ( 1.0 / 8.0 );
  L_Center  :=   Rand(X  , Y  ) * ( 1.0 / 4.0 );

  result := L_Center + L_Sides + L_Corners;}
end;

function TPerlinNoise2D.Rand(X, Y: Integer): Single;
var
  L_Base: Integer;
begin
  L_Base := X + Y * 57 + m_Seed;
  L_Base := (L_Base shl 13) xor L_Base;
  result := ( 1 - ( (L_Base * (L_Base * L_Base * 15731 + 789221) + 1376312589) and $7fffffff) * ( 1 / 1073741824 ));
end;

end.